@@ -1,46 +1,25 @@
 
 #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-# Functions
+# S3 methods ###################################################################
 #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-#' @title 
-#' Perform ridge regression on scaled expression data
-#' 
-#' @description 
-#' Perform ridge regression on scaled expression data, accepting both technical 
-#' and biological categorical variables. The effect of the technical variables 
-#' is removed while the effect of the biological variables is retained. This is 
-#' a preprocessing step that can aid BBKNN integration.
-#' 
-#' @param object An object
-#' @param ... Arguments passed to other methods
-#' 
-#' @references Park, Jong-Eun, et al. "A cell atlas of human thymic development defines T 
-#' cell repertoire formation." Science 367.6480 (2020): eaay3224.
-#' 
-#' @name RidgeRegression
-#' 
-#' @export
-RidgeRegression <- function(object, ...) {
-  UseMethod(generic = 'RidgeRegression', object = object)
-}
-
 #' @param latent_data Extra data to regress out, should be cells x latent data
-#' @param batch_key Variables to regress out as technical effects. Must be included in column 
-#' names of latent_data
-#' @param confounder_key Variables to to retain as biological effects. Must be included in 
-#' column names of latent_data
-#' @param lambda A user supplied lambda sequence. pass to \code{\link[glmnet]{glmnet}}
-#' @param seed Set a random seed. By default, sets the seed to 42. Setting NULL will not set 
-#' a seed.
+#' @param batch_key Variables to regress out as technical effects. Must be 
+#' included in column names of latent_data
+#' @param confounder_key Variables to to retain as biological effects. Must be 
+#' included in column names of latent_data
+#' @param lambda A user supplied lambda sequence. pass to 
+#' \code{\link[glmnet]{glmnet}}
+#' @param seed Set a random seed. By default, sets the seed to 42. Setting NULL 
+#' will not set a seed.
 #' @param verbose Whether or not to print output to the console
 #' 
-#' @import glmnet
+#' @importFrom glmnet glmnet coef.glmnet
 #' @importFrom tidytable get_dummies.
 #' 
 #' @rdname RidgeRegression
-#' @method RidgeRegression default
 #' @export
+#' @method RidgeRegression default
 RidgeRegression.default <- function(
   object,
   latent_data,
@@ -53,7 +32,7 @@ RidgeRegression.default <- function(
 ) {
   latent_data <- latent_data[, c(batch_key, confounder_key), drop = FALSE]
   if (verbose) {
-    cat("Running ridge regression...\n")
+    message("Running ridge regression...")
   }
   if (!is.null(x = seed)) {
     set.seed(seed = seed)
@@ -92,22 +71,28 @@ RidgeRegression.default <- function(
 }
 
 #' @param assay Name of Assay ridge regression is being run on
-#' @param features Features to compute ridge regression on. If features=NULL, ridge regression 
-#' will be run using the variable features for the Assay. 
-#' @param run_pca Whether or not to run pca with regressed expression data (TRUE by default)
+#' @param features Features to compute ridge regression on. If features=NULL, 
+#' ridge regression will be run using the variable features for the Assay. 
+#' @param run_pca Whether or not to run pca with regressed expression data 
+#' (TRUE by default)
 #' @param npcs Total Number of PCs to compute and store (50 by default)
 #' @param reduction.name Dimensional reduction name (pca by default)
-#' @param reduction.key Dimensional reduction key, specifies the string before the number for the 
-#' dimension names (PC by default)
-#' @param replace Whether or not to replace original scale.data with regressed expression data (TRUE by default)
+#' @param reduction.key Dimensional reduction key, specifies the string before 
+#' the number for the dimension names (PC by default)
+#' @param replace Whether or not to replace original scale.data with regressed 
+#' expression data (TRUE by default)
 #' 
 #' @return Returns a Seurat object.
 #' 
-#' @import Seurat SeuratObject
+#' @importFrom Seurat RunPCA
+#' @importFrom SeuratObject DefaultAssay FetchData GetAssayData SetAssayData 
+#' VariableFeatures
+#' @importFrom utils packageVersion
+#' @importClassesFrom SeuratObject Seurat
 #' 
 #' @rdname RidgeRegression
-#' @method RidgeRegression Seurat
 #' @export
+#' @method RidgeRegression Seurat
 RidgeRegression.Seurat <- function(
   object,
   batch_key,
@@ -127,13 +112,29 @@ RidgeRegression.Seurat <- function(
   assay <- assay %||% DefaultAssay(object = object)
   features <- features %||% VariableFeatures(object = object, assay = assay)
   if (!any(c(run_pca, replace))) {
-    warning("At least one of 'run_pca' or 'replace' should be set up.\nReturn the original object", immediate. = TRUE)
+    warning(
+      "At least one of 'run_pca' or 'replace' should be set up.\n", 
+      "Return the original object", 
+      immediate. = TRUE
+    )
     return(object)
   }
-  data.expr <- GetAssayData(object = object, assay = assay, slot = "scale.data")
+  if (packageVersion(pkg = "Seurat") >= package_version(x = "5.0.0")) {
+    data.expr <- GetAssayData(
+      object = object, 
+      assay = assay, 
+      layer = "scale.data"
+    )
+  } else {
+    data.expr <- GetAssayData(
+      object = object, 
+      assay = assay, 
+      slot = "scale.data"
+    )
+  }
   features <- intersect(x = features, y = rownames(x = data.expr))
   data.expr <- data.expr[features, ]
-  latent_data <- object@meta.data[, c(batch_key, confounder_key), drop = FALSE]
+  latent_data <- FetchData(object = object, vars = c(batch_key, confounder_key))
   data.resid <- RidgeRegression(
     object = data.expr,
     latent_data = latent_data,
@@ -146,13 +147,27 @@ RidgeRegression.Seurat <- function(
   )
   if (replace) {
     if (verbose) {
-      cat("Replace original scale.data...\n")
+      message("Replace original scale.data...")
+    }
+    if (packageVersion(pkg = "Seurat") >= package_version(x = "5.0.0")) {
+      object <- SetAssayData(
+        object = object, 
+        assay = assay, 
+        layer = "scale.data", 
+        new.data = data.resid
+      )
+    } else {
+      object <- SetAssayData(
+        object = object, 
+        assay = assay, 
+        slot = "scale.data", 
+        new.data = data.resid
+      )
     }
-    object <- SetAssayData(object = object, assay = assay, slot = "scale.data", new.data = data.resid)
   }
   if (run_pca) {
     if (verbose) {
-      cat("Running PCA...\n")
+      message("Running PCA...")
     }
     pca <- RunPCA(
       object = data.resid, 
